import { useRef, useEffect, useState } from 'react';
import { ChatbotWebSocket } from '../providers';
import { From } from '../types';
import { useStorage } from '../context/Storage/Component';
import { useSocketActions } from './useSocketActions';
import { sendNotification } from '../utils';
import { useEncryptedStorage } from './useEncryptedStorage';
import { Global } from '../global';
export const useSocket = wsUrl => {
  const {
    retrieve
  } = useEncryptedStorage();
  const isFirstRender = useRef(true);
  const wsRef = useRef(null);
  const [waitingToReconnect, setWaitingToReconnect] = useState(false);
  const {
    subscribe
  } = useSocketActions();
  const {
    addMessage
  } = useStorage();
  const onOpenCallback = async () => {
    const sessionId = await retrieve('sessionId');
    if (!Global.socketId) {
      var _wsRef$current;
      (_wsRef$current = wsRef.current) === null || _wsRef$current === void 0 ? void 0 : _wsRef$current.sendMessage({
        action: 'link',
        data: {
          sessionId,
          botId: Global.botId
        }
      });
    }
  };
  const onMessageCallback = msg => {
    sendNotification(msg);
    addMessage({
      ...msg,
      from: From.BOT
    });
  };
  const disconnect = () => {
    var _wsRef$current2;
    (_wsRef$current2 = wsRef.current) === null || _wsRef$current2 === void 0 ? void 0 : _wsRef$current2.disconnect();
    setWaitingToReconnect(true);
  };
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      return;
    }
    if (!wsRef.current) {
      wsRef.current = new ChatbotWebSocket(wsUrl);
      wsRef.current.onMessage(onMessageCallback);
      wsRef.current.onOpen(onOpenCallback);
      wsRef.current.connect();
    }
    return () => {
      var _wsRef$current3;
      (_wsRef$current3 = wsRef.current) === null || _wsRef$current3 === void 0 ? void 0 : _wsRef$current3.disconnect();
    };
  }, [isFirstRender.current, waitingToReconnect]);
  useEffect(() => {
    if (isFirstRender.current) {
      isFirstRender.current = false;
      return;
    }
    (async () => {
      if (Global.socketId) await subscribe();
    })();
  }, [isFirstRender.current, Global.socketId]);
  return {
    disconnect
  };
};
//# sourceMappingURL=useSocket.js.map